% po.sl -*- mode: slang; mode: fold -*-
%%
%% Editing mode for gettext portable object files (*.po)
%%
%% To use it, insert the following lines into your .jedrc:
%%    autoload ("po_mode", "po");
%%    add_mode_for_extension ("po", "po");
%%    add_mode_for_extension ("po", "pot");
%%
%%
%{{{ user configurable items, colors, email, language-team
%
%%% CONFIGURABLE ITEMS - edit the lines below to reflect your settings. %%%
%% (Look toward the end of the file for the keymaps in order to customize
%% the keys).
%% 
%% Translator's name:
%% 
static variable name = "Morten Bo Johansen";
%%
%% Translator's email address:
%% 
static variable email = "mj@mail.dk";
%%
%% Language Team of translator:
%% 
static variable team = "Danish <dansk@klid.dk>";
%%
%% Default character set in po-header:
%% 
static variable charset = "ISO-8859-1";
%%
%% If po-file has plural forms: how many plural forms does your language
%% have? See /usr/share/doc/gettext-doc/gettext_10.html#SEC1.
%% 
static variable nplurals = "2";
%%
%% Plural expression (see doc):
%% 
static variable plural = "n != 1";
%%
%% Should newline characters be represented by "\n"?:
%% 0 = no, 1 = yes
%% 
static variable newline_chars = 0;
%%
%% Set the revision date of po-file upon exit?:
%% 0 = never, 1 = ask, 2 = always
%% 
static variable revise_date = 1;
%%
%% Use DFA syntax highlighting? Adjust the colors below and compile the
%% table -- see the file, dfa.txt, in the jed archive for an explanation.
%% 0 = no, 1 = yes
%% 
static variable use_dfa = 1;
%%
%% The colors below are based on the "white1" and "elegant" color schemes for
%% jed and xjed respecetively. You may want to change them to suit your color
%% schemes of choice - or disable dfa highligting.

%% colors for jed/console
%% 
%% "black"     "gray"
%% "red"       "brightred"
%% "green"     "brightgreen"
%% "brown"     "yellow"
%% "blue"      "brightblue"
%% "magenta"   "brightmagenta"
%% "cyan"      "brightcyan"
%% "lightgray" "white"

static variable bg = "white";             % background color

set_color ("keyword1","red",bg);          % fuzzy, c-format
set_color ("keyword2","magenta",bg);      % msgid/msgstr labels
set_color ("keyword3","black","cyan");    % spaces at beg./end of msgid/msgstr
set_color ("keyword4","blue",bg);         % "\n"
set_color ("keyword5","gray",bg);         % source code reference
set_color ("keyword6","black","yellow");  % translators' comments

%% colors for xjed. You can use the whole slew of colors that are
%% available under X, see /usr/X11R6/lib/X11/rgb.txt for some of
%% them.
%%
if (NULL != getenv ("DISPLAY"))
{
   static variable bg = "#E0E0E0";

   set_color ("keyword1","DarkRed",bg);
   set_color ("keyword2","magenta",bg);
   set_color ("keyword3","black","cyan");
   set_color ("keyword4","blue",bg);
   set_color ("keyword5","DarkGray",bg);
   set_color ("keyword6","black","yellow");
}

%%% END CONFIGURABLE ITEMS %%%
%}}}

%{{{ dfa syntax

create_syntax_table ("PO");

#ifdef HAS_DFA_SYNTAX
%%% DFA_CACHE_BEGIN %%%
static define setup_dfa_callback (name)
{
   variable autocomment = "keyword5";
   variable flagcomment = "keyword1";
   variable srccomment = "keyword5";
   variable usercomment = "keyword6";
   variable strings = "keyword2";
   variable begblank = "keyword3";
   variable endblank = "keyword3";
   variable begblank_editbuf = "keyword3";
   variable endblank_editbuf = "keyword3";
   variable lbreak = "keyword4";

   dfa_enable_highlight_cache("po.dfa", name);
   dfa_define_highlight_rule("^#[,]+.*", flagcomment, name);
   dfa_define_highlight_rule("^#[:]+.*", srccomment, name);
   dfa_define_highlight_rule("^#[\\.]+.*", autocomment, name);
   dfa_define_highlight_rule("^#[ \t]+.*", usercomment, name);
   dfa_define_highlight_rule("^[ \t]*msg(id|str)", strings, name);
   dfa_define_highlight_rule("[ \t]*\"[ \t]+", begblank, name);   % initial blank was needed
   dfa_define_highlight_rule("^[ \t]+", begblank_editbuf, name);
   dfa_define_highlight_rule("[ \t]+$", endblank_editbuf, name);
   dfa_define_highlight_rule("[ \t]+\\\\?n?\"$", endblank, name);
   dfa_define_highlight_rule("\\\\n", lbreak, name);
   dfa_build_highlight_table(name);
}
dfa_set_init_callback (&setup_dfa_callback, "PO");
%%% DFA_CACHE_END %%%
#endif
%}}}

%{{{ delimitation

static define is_header ()
{
   push_spot ();
   EXIT_BLOCK
     {
        pop_spot ();
     }
   forever
     {
        if ( orelse 
             { bol_fsearch ("\"Project-Id-Version") }
               { bol_fsearch ("\"POT-Creation-Date") }
               { bol_fsearch ("\"PO-Revision-Date") }
               { bol_fsearch ("\"Last-Translator") }
               { bol_fsearch ("\"Language-Team") }
               { bol_fsearch ("\"MIME-Version") }
               { bol_fsearch ("\"Content-Type") }
               { bol_fsearch ("\"Content-Transfer-Encoding") }
               { bol_fsearch ("\"Plural-Forms") }
             )
          return 1;
        else return 0;
     }
}

static define is_translator_comment ()
{
   bol ();
   if ( orelse 
        { looking_at ("# ") }
          { looking_at ("# \t") }
          { looking_at ("#\t ") }
          { looking_at ("#\n") }
        )
     return 1;
   else return 0;
}

static define is_other_comment ()
{
   bol ();
   if ( orelse 
        { looking_at ("#:") }
          { looking_at ("#.") }
          { looking_at ("#,") }
        )
     return 1;
   else return 0;
}

%% Comment type for obsolete entries #~ should not be included
%% 
define is_on_delimiter ()
{
   bol (); skip_white ();
   return eolp () or is_translator_comment () or is_other_comment ();
}

static define is_last_entry ()
{
   push_spot ();
   EXIT_BLOCK
     {
        pop_spot ();
     }
   if (is_on_delimiter ()) re_fsearch ("[#~ ]*msgid"); down_1;
   do
     {
        if (re_fsearch ("[#~ ]*msgid")) break;
     }
   while (down_1);
   return eobp ();
}

%% Should ensure correct identification of untranslated plural msgstrs 
%% and of such that are not preceded by a delimiter (blank line or comment)
%% 
static define is_untranslated ()
{
   push_spot();
   EXIT_BLOCK 
     {
        pop_spot();
     }
   
   () = fsearch ("\"");
   !if (looking_at ("\"\""))
     return 0;
   go_down_1 ();
   if (looking_at ("msgstr[")) return 1;
   else
     {
        skip_chars (" \t\n");
        return looking_at ("#") or looking_at ("msgid") or eobp ();
     }
}

static define is_obsolete ()
{
}

%% Determine the span of the entry in a predictable way regardless
%% of the position of editing point and of different formats of the
%% entries.
%% If standing on a blank line between blocks of msgid/msgstr then   
%% advance to next block otherwise operate on current block.
%% Gettext's parsing rules make both source reference comments, 
%% other comments and blank lines between blocks of strings optional.
%% Also allow editing of obsolete entries like emacs' po-mode.
%% 
define narrow_to_strings ()
{          
   if (is_last_entry)
     {
        eob ();
        re_bsearch ("[~# ]*msgid "); % final space is to skip msgid_plural
        push_mark ();
        eob ();
     }
   else if (is_on_delimiter ())
     {                             
        re_fsearch ("^[#~ ]*msgid ");
        push_mark ();
        go_down_1 ();
        re_fsearch ("^[#~ ]*msgid ");
        go_up_1 (); bol ();
        while (looking_at ("#"))
          { 
             go_up_1 (); bol ();
          }
        eol ();
      }
   else
     {
        !if (looking_at ("msgid ") or looking_at ("#~ msgid "))
          re_bsearch ("^[#~ ]*msgid ");
        push_mark ();
        go_down_1 ();
        re_fsearch ("^[#~ ]*msgid ");
        go_up_1 (); bol ();
        while (looking_at ("#"))
          { 
             go_up_1 (); bol ();
          }
        eol ();
     }
   narrow_to_region ();
   bob ();
}

define is_msgid_plural ()
{
   push_spot ();
   narrow_to_strings ();
   !if (fsearch ("msgid_plural"))
     {
        widen_region ();
        return 0;
     }
   widen_region ();
   pop_spot ();
}

%% Like narrow_to_strings () but include translator's comments and other
%% comments as well.
%% 
define narrow_to_whole_block ()
{
   narrow_to_strings ();
   widen_region ();
   if (is_header ())
     {
        bob (); push_mark ();
        while (is_header ()) go_down_1 ();
        narrow_to_region ();
        bob ();
        return;
     }
   push_spot ();
   go_up_1 (); bol ();
   !if (is_on_delimiter ())
     {
        pop_spot (); push_mark (); go_down_1 ();
        re_fsearch ("^[#~ ]*msgid ");
        go_up_1 (); bol ();
        while (looking_at ("#"))
          { 
             go_up_1 (); bol (); 
          }
        eol ();
     }
   else
     {
        pop_spot ();
        narrow_to_strings ();
        eob (); push_mark (); bob ();
        widen_region ();
        go_up_1 (); bol ();
        while (looking_at ("#"))
          { 
             go_up_1 (); bol ();
          }
        go_down_1 ();
     }
   narrow_to_region ();
   bob ();
}

define mark_translator_comment ()
{
   narrow_to_whole_block ();
   push_spot ();
   while (fsearch ("#") and is_other_comment ()) go_down_1 ();
   if (is_translator_comment ())
     {
        push_mark ();
        while (is_translator_comment ()) go_down_1 ();
     }
   else
     {
        pop_spot ();
        push_mark ();
     }
   widen_region ();
}

define mark_msgid ()
{
   if (is_msgid_plural ())
     {
        narrow_to_strings ();
        push_mark ();
        bol_fsearch ("msgstr");
     }
   else 
     {
        narrow_to_strings ();
        fsearch ("\"");
        !if (looking_at ("\"\""))
          push_mark_eol ();
        else
          {
             go_down_1 (); push_mark ();
             re_fsearch ("^[#~ ]*msgstr");
             bsearch ("\"");
             go_right_1 ();
          }
     }
   widen_region ();
}

define mark_msgstr ()
{
   narrow_to_strings ();
   if (re_fsearch ("^msgstr[[]+")) % plural msgstrs
     {
        variable n = read_mini("Enter number of msgstr to edit:", "", "");
        if (n == "0")
          {
             fsearch ("\"");
             push_mark ();
             bol_fsearch ("msgstr");
          }
        else
          {  
             re_fsearch ("^msgstr[[]+" + string(n));
             fsearch ("\"");
             push_mark ();
             !if (bol_fsearch ("msgstr"))
               eob ();
             else bol_fsearch ("msgstr");
          }
     }
   else
     {
        re_fsearch ("^[#~ ]*msgstr");
        fsearch ("\"");
        push_mark (); eob ();
     }
   bskip_chars ("\n");
   widen_region ();
}

define del_msgstr ()
{
   mark_msgstr ();
   del_region ();
}

%}}}

%{{{ po-header

%% The default header from emacs' po-mode
%% 
static define insert_po_header ()
{
   set_readonly (0);

   insert 
     ("# SOME DESCRIPTIVE TITLE.\n" +
      "# Copyright (C) YEAR Free Software Foundation, Inc.\n" +
      "# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.\n" +
      "#\n" +
      "#, fuzzy\n" +
      "msgid \"\"\n" +
      "msgstr \"\"\n" +
      "\"Project-Id-Version: PACKAGE VERSION\\n\"\n" +
      "\"PO-Revision-Date: YEAR-MO-DA HO:MI +ZONE\\n\"\n" +
      "\"Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n\"\n" +
      "\"Language-Team: LANGUAGE <LL@li.org>\\n\"\n" +
      "\"MIME-Version: 1.0\\n\"\n" +
      "\"Content-Type: text/plain; charset=CHARSET\\n\"\n" +
      "\"Content-Transfer-Encoding: 8bit\\n\"\n\n");

   bob ();
   set_readonly (1);
}

%% If file is empty then insert a header. No sense with it for the time being.
%% 
if (bobp () and eobp ())
  insert_po_header ();


%% Check if a po-header is present
%% 
static define check_header ()
{
   push_spot ();
   set_readonly (0);
   bob ();
   !if (is_header)
     {
        switch (get_mini_response ("No header found? Insert one? [y/n]: "))
          { case 'y': insert_po_header (); }
          { case 'n': return; }
     }
   set_readonly (1);
   pop_spot ();
}

%% Adjust the po-header to user's settings.
%% 
define replace_headers ()
{
   push_spot ();
   check_header ();
   set_readonly (0);
   variable t, n, c, p;
   t = ("\"Language-Team: " + team + "\\n\"");
   n = ("\"Last-Translator: " + name + " " +  "<" + email + ">" + "\\n\"");
   c = ("\"Content-Type: text/plain; charset=" + charset + "\\n\"");
   p = ("\"Plural-Forms: nplurals=" + nplurals + "; " + "plural=" + plural +
        ";" + "\\n\"");
   
   bob ();
   re_fsearch ("\"Last-Translator.*");
   replace_match (n, 1);
   re_fsearch ("\"Language-Team.*");
   replace_match (t, 1);
   bob ();
   re_fsearch ("\"Content-Type.*");
   replace_match (c, 1);
   bob ();
   if (re_fsearch ("\"Plural-Forms.*"))
     replace_match (p, 1);
   pop_spot ();
   set_readonly (1);
}

%% Set the revision date for the po-file.
%% 
define set_po_revision_date ()
{
   set_readonly (0);
   variable gm, gmhour, lm, lmhour, offset;
   variable tm, day, month, year;
   tm = localtime (_time ());
   gm = (gmtime (_time ()));
   gmhour = gm.tm_hour;
   lm = (localtime (_time ()));
   lmhour = lm.tm_hour;
   offset = (lmhour-gmhour)*100;
   variable t = sprintf ("%d-%02d-%02d %02d:%02d", 1900 + tm.tm_year, 1+tm.tm_mon, tm.tm_mday, tm.tm_hour, tm.tm_min);
   bob ();
   fsearch ("PO-Revision-Date");
   fsearch (":");
   go_right_1 (); push_mark_eol (); 
   del_region (); 
   whitespace (1);
   vinsert ("%s+%04d%s", t, offset, "\\n\"");
   set_readonly (1);
}

define set_date_on_exit ()                                                       
{                                                                         
   check_header ();
   if (revise_date == 1)
     {
        switch (get_mini_response ("May I set the PO Revison date?: [y/n]"))
          { case 'y': set_po_revision_date (); }
          { case 'n'; return; }
     }
   if (revise_date == 2) 
     {
        set_po_revision_date (); 
     }
}             

%}}}

%{{{ find functions

define goto_next_entry ()
{
   narrow_to_strings ();
   eob ();
   widen_region ();
   if (0 == re_fsearch ("^[#~ ]*msgid"))
     {
        error ("You are on the last entry");
        return;
     }
}

define goto_previous_entry ()
{
   narrow_to_strings ();
   bob ();
   widen_region ();
   if (0 == re_bsearch ("^[#~ ]*msgid"))
     {
        error ("You are on the first entry");
        return;
     }
}

%% Find next untranslated msgstr and wrap search around buffer
%% The wrapping could probably be done somewhat more elegant?
%% 
define find_untranslated () 
{
   push_mark ();
   if (is_header) goto_next_entry ();
   while (re_fsearch ("^msgstr[[]*"))
     if (is_untranslated () and fsearch ("\""))
       { 
          pop_mark (0);
          return;
       }
   bob ();
   goto_next_entry ();
   while (re_fsearch ("^msgstr[[]*"))
     if (is_untranslated () and fsearch ("\""))
       {
          pop_mark (0);
          return;
       }
   pop_mark (1);
   flush ("No untranslated strings");
   return;
}

%% Find the previous untranslated msgstr.
%%
define bfind_untranslated () 
{
   push_mark();
   while (re_bsearch ("^msgstr[[]*"))
     if (is_untranslated ())
       {
          pop_mark (0);
          return;
       }
   pop_mark (1);
   flush ("Not found");
   return;
}

%% Find the next translated msgstr and wrap search around buffer
%% 
define find_translated () 
{
   push_mark ();
   if (is_header) goto_next_entry ();
   while (re_fsearch ("^msgstr[[]*"))
     if (not (is_untranslated ()) and fsearch ("\""))
       { 
          pop_mark (0);
          return;
       }
   bob ();
   goto_next_entry ();
   while (re_fsearch ("^msgstr[[]*"))
     if (not (is_untranslated ()) and fsearch ("\""))
       {
          pop_mark (0);
          return;
       }
   pop_mark (1);
   flush ("No translated strings");
   return;
}

%% Find the previous translated msgstr.
%% 
define bfind_translated () 
{
   push_mark ();
   while (re_bsearch ("^msgstr[[]*"))
     if (not (is_untranslated ()) and bsearch ("\""))
       {
          bol ();
          pop_mark (0);
          return;
       }
   flush ("Not found");
   pop_mark (1);
   return;
}

%% Find the next entry with fuzzy flag and wrap search around buffer.
%% 
define find_fuzzy ()
{
   while (re_fsearch ("^#,[ \t]*fuzzy"))
     {
        re_fsearch ("^[#~ ]*msgid");
        return;
     }
   push_mark ();
   bob ();
   while (re_fsearch ("^#,[ \t]*fuzzy"))
     {
        re_fsearch ("^[#~ ]*msgid");
        pop_mark (0);
        return;
     }
   pop_mark (1);
   flush ("No more fuzzy strings");
}

%% Find the previous entry with fuzzy flag.
%% 
define bfind_fuzzy ()
{
   while (re_bsearch ("^#,[ \t]*fuzzy")) return;
   push_mark ();
   bob ();
   while (re_bsearch ("^#,[ \t]*fuzzy"))
     {
        pop_mark (0);
        return;
     }
   pop_mark (1);
   flush ("Not found");
}

%% Find the next obsolete entry and wrap search around buffer.
%% 
define find_obsolete ()
{
   while (bol_fsearch ("#~"))
     {
        fsearch ("msgstr");
        return;
     }
   push_mark ();
   bob ();
   while (bol_fsearch ("#~"))
     {
        fsearch ("msgstr");
        pop_mark (0);
        return;
     }
   pop_mark (1);
   flush ("No more obsolete entries");
}

%% Find the previous obsolete entry.
%% 
define bfind_obsolete ()
{
   while (bol_bsearch ("#~"))
     { 
        bsearch ("msgid");
        return;
     }
   push_mark ();
   bob ();
   while (bol_bsearch ("#~"))
     {
        bsearch ("msgid");
        pop_mark (0);
        return;
     }
   pop_mark (1);
   flush ("Not found");
}

%}}}

%{{{ modification of entries outside of edit buffer

%% Remove fuzzy flag from entry.
%% 
define del_fuzzy ()
{
   set_readonly (0);
   narrow_to_whole_block ();
   !if (re_fsearch("#,[ \t]*fuzzy"))
     {
        widen_region ();
        error ("Entry is not marked fuzzy");
     }
   delete_line ();
   widen_region ();
   set_readonly (1);
}

define del_translator_comment ()
{
   set_readonly (0);
   mark_translator_comment ();
   del_region ();
   set_readonly (1);
}

define del_obsolete ()
{
   set_readonly (0);
   narrow_to_whole_block ();
   if (fsearch ("#~"))
     {
        mark_buffer ();
        del_region ();
        widen_region ();
     }
   else 
     {
        widen_region ();
        error ("Not an obsolete entry");
     }
   set_readonly (1);
}

%% Flag translation as "fuzzy".
%% 
define fuzzyize ()
{
   set_readonly (0);
   narrow_to_whole_block ();
   if (re_fsearch ("^#,[ \t]*fuzzy"))
     {
        widen_region ();
        error ("String is already marked fuzzy");
     }
   widen_region ();
   narrow_to_strings ();
   newline ();
   up_1;
   insert ("#, fuzzy");
   widen_region ();
   set_readonly (1);
}

%% Make entry obsolete.
%% 
define obsolete_entry ()
{
   set_readonly (0);
   push_spot ();
   narrow_to_whole_block ();
   if (bol_fsearch ("#~")) 
     {
        widen_region ();
        error ("Entry already obsolete");
     }
   else
     {
        do
          {
             if (eolp () and bolp ()) break;
             insert ("#~ ");
          }
        while (down (1));
        widen_region ();
        pop_spot ();
     }
   set_readonly (1);
}

%% Remove contents of msgstr (leaving it unstranslated)
%% 
define cut_msgstr ()
{
   set_readonly (0);
   del_msgstr ();
   insert ("\"\"");
   set_readonly (1);
}

%% Copy contents of msgid to msgstr
%% 
define copy_msgid_to_msgstr ()
{
   set_readonly (0);
   mark_msgid ();
   narrow_to_region ();
   mark_buffer ();
   variable n = what_line ();
   variable msgid = bufsubstr ();
   widen_region ();
   del_msgstr ();
   push_spot ();
   if (n > 1)
     {
        insert ("\"\"");
        down_1;
        insert (msgid);
        newline ();
     }
   else insert (msgid);
   pop_spot ();
   set_readonly (1);
}

define copy_msgstr ()
{
   push_spot (); 
   mark_msgstr ();
   yp_copy_region_as_kill ();
   pop_spot ();
}

define insert_msgstr ()
{
   set_readonly (0);
   push_spot ();
   del_msgstr ();
   yp_yank ();
   pop_spot ();
   set_readonly (1);
}

define edit_whole_buffer ()
{
   no_mode ();
   use_keymap ("global");
   toggle_readonly ();
   call ("redraw");
   flush ("Use \"M-x po_mode\" to return");
}

define po_undo ()
{
   set_readonly (0);
   call ("undo");
   set_readonly (1);
}

%}}}

%{{{ po_edit buffer

%% All functions herein apply to the temporary editing buffer.
%%
%% Mode for the editing buffer.
%% 
static define po_edit_mode ()
{
   set_mode ("po_edit", 0);
   use_keymap ("po_edit");
   use_syntax_table ("PO");
   use_dfa_syntax (use_dfa);
   if (abbrev_table_p ("po_edit")) use_abbrev_table ("po_edit");
   set_abbrev_mode (1);
}

static define po_comment_mode ()
{
   set_mode ("po_comment", 0);
   use_keymap ("po_comment");
}

define decreasewin ()
{
   otherwindow ();
   enlargewin ();
   otherwindow ();
}

custom_variable ("newline_seq", newline_chars);

%% Should newline characters be substituted by the C-like escape
%% sequence "\n".
%% 
define newline_seq_on ()
{
   if (newline_seq == 0)
     {
        newline_seq = 1;
        flush ("Newline characters will be represented by \"\\n\"");
     }
   else return;
}

define newline_seq_off ()
{
   if (newline_seq == 1)
     {
        newline_seq = 0;
        flush ("Normal handling of newline characters restored");
     }
   else return;
}

static define strip_quotes ()
{
   bob ();
   if (looking_at ("\"\"")) delete_line ();
   if (bobp () and eobp ()) return;
   do
     {
        bol (); del ();
        eol (); go_left_1; del ();
     }
   while (down (1) and not eobp);
}

%% De-escape double quotes before inserting msgstr into edit buffer
%% 
static define non_esc_quotes ()
{
   bob ();
   replace ("\\\"", "\"");
}

static define strip_newline_seq ()
{
   if (fsearch ("\\n"))
     replace ("\\n", Null_String);
}

static define trim_blank_lines ()  % no blank lines after msgstr
{
   eob ();
   bskip_chars ("\n");
   if (eobp () and eolp ())
     return;
   else
     {
        eob ();
        bskip_chars ("\n");
        do
          {
             del ();
          }
        while (not eobp ());
     }
}

%% Escape double quotes before merging edit buffer with po-file
%% 
static define esc_quotes ()
{
   bob ();
   replace ("\"", "\\\"");
}

%% Surround all msgstr lines in double quotes
%% 
static define add_quotes ()
{
   bob ();
   do
     {
        bol (); insert ("\"");
        eol (); insert ("\"");
     }
   while (down_1 ());
}

static define insert_newline_seq ()
{
   bob ();
   replace ("\n", "\\n\n");
}

define insert_lbreak ()
{
   insert ("\\n\n");
}
     
static variable editbuf = "*"+whatbuf ()+"*";

%% Mark contents of msgid and narrow them in a window on top of screen
%% put contents of current msgstr stripped of surrounding  quote characters 
%% in its own temporary editing buffer underneath window of msgid
%%
define edit ()
{
   set_readonly (0);
   variable msgstr = "";
   mark_msgstr ();
   msgstr = bufsubstr_delete ();
   push_spot ();
   mark_msgid ();
   narrow_to_region ();
   recenter (1);
   pop2buf (editbuf);
   insert (msgstr);
   strip_quotes ();
   non_esc_quotes ();
   if (newline_seq == 1)
     strip_newline_seq ();
   otherwindow ();
   fit_window ();
   bob ();
   otherwindow ();
   bob ();
   po_edit_mode ();
   set_buffer_undo (1);
}

%% Read contents of temporary msgstr editing buffer back into po-file, 
%% adding quotes and trimming blank lines. Before doing so get length
%% and number of lines of new msgstr and decide how to place it based 
%% on that. Escape manually entered double quotes before surrounding 
%% all lines in double quotes.
%%
define end_edit ()
{
   variable buf, len, n;
   trim_blank_lines ();
   buf = "";
   mark_buffer ();
   n = what_line ();
   buf = bufsubstr ();
   len = strlen (buf);
   if (newline_seq == 1)
     insert_newline_seq ();
   esc_quotes ();
   add_quotes ();
   set_buffer_modified_flag (0);
   otherwindow ();
   onewindow ();
   widen_region ();
   if (len > 71 or n > 1)
     {
        pop_spot ();
        insert ("\"\"");
        down_1;
        insbuf (editbuf);
        newline ();
     }
   else
     {
        pop_spot ();     
        insbuf (editbuf);
     }
   bol_bsearch ("msgid");
   delbuf (editbuf);
   goto_next_entry ();
   set_readonly (1);
}

%% Translator's comments:
%% 

static define strip_comments ()
{
   bob ();
   while (is_translator_comment)
     {
        do
          {
             bol (); push_mark ();
             skip_chars ("# \t");
             del_region ();
          }
        while (down_1);
     }
}

define add_comments ()
{
   if (eobp () and bobp ()) return;
   bob ();
   do
     {
        bol (); insert ("# ");
     }
   while (down (1));
   eob ();
   newline ();
}

define edit_comment ()
{
   variable comment = "";
   set_readonly (0);
   mark_translator_comment ();
   comment = bufsubstr_delete ();
   push_spot ();
   narrow_to_whole_block ();
   recenter (1);
   pop2buf (editbuf);
   insert (comment);
   strip_comments ();
   otherwindow ();
   fit_window ();
   bob ();
   otherwindow ();
   bob ();
   po_comment_mode ();
}

define end_edit_comment ()
{
   trim_blank_lines ();
   variable buf = "";
   mark_buffer ();
   buf = bufsubstr ();
   add_comments ();
   set_buffer_modified_flag (0);
   otherwindow ();
   onewindow ();
   widen_region ();
   pop_spot ();
   insbuf (editbuf);
   delbuf (editbuf);
   set_readonly (1);
}

%}}}

%{{{ msgfmt functions

define po_validate_command ()
{
   compile ("msgfmt --statistics -c -v -o /dev/null " + whatbuf());
}

define po_compile ()
{
   variable dir = dircat (getenv ("HOME"), Null_String);
   variable str = read_mini ("Compile command: ", "", "msgfmt" +
                             " -o " + dir + path_sans_extname (whatbuf()) +
                             ".mo " + whatbuf());
   if (strlen (str))
     compile (str);
}

%% Display some statistics about the po-file to the user.
%% 
define po_statistics ()
{
   variable t = 0, u = 0, f = 0, o = 0;
   push_spot_bob ();
   if (is_header) goto_next_entry ();
   while (re_fsearch ("msgstr[[]*"))
     {
        if (is_untranslated () and fsearch ("\"")) u++;
     }
   bob ();
   if (is_header) goto_next_entry ();
   while (re_fsearch ("msgstr[[]*"))
     {
        if (not (is_untranslated ()) and fsearch ("\"")) t++;
     }
   bob ();
   while (re_fsearch ("^#,[ \t]*fuzzy"))
     { 
        go_down_1 (); 
        f++;
     }
   bob ();
   while (re_fsearch ("^#~[ \t]*msgid"))
     { 
        go_down_1 ();
        o++;
     }
   vmessage ("%d translated, %d untranslated, %d fuzzy, %d obsolete", 
             t, u, f, o);
   pop_spot ();
}

%}}}

%{{{ keymaps, hooks, menu, mode definition


Help_File = "po.hlp";

static define po_init_menu (menu)
{
   menu_append_item (menu, "Undo",  "po_undo");
   menu_append_item (menu, "Statistics",  "po_statistics");
   menu_append_item (menu, "Newlines on",  "newline_seq_on");
   menu_append_item (menu, "Newlines off",  "newline_seq_off");
   menu_append_popup (menu, "&Navigate");
   $0 = menu + ".&Navigate";
     {
        menu_append_item ($0, "Next entry",  "goto_next_entry");
        menu_append_item ($0, "Previous entry",  "goto_previous_entry");
        menu_append_item ($0, "Next untranslated",  "find_untranslated");
        menu_append_item ($0, "Previous untranslated",  "bfind_untranslated");
        menu_append_item ($0, "Next translated",  "find_translated");
        menu_append_item ($0, "Previous translated",  "bfind_translated");
        menu_append_item ($0, "Next fuzzy",  "find_fuzzy");
        menu_append_item ($0, "Previous fuzzy",  "bfind_fuzzy");
        menu_append_item ($0, "Next obsolete",  "find_obsolete");
        menu_append_item ($0, "Previous obsolete",  "bfind_obsolete");
     }
   menu_append_popup (menu, "&Edit");
   $1 = menu + ".&Edit";
     {
        menu_append_item ($1, "Edit msgstr",  "edit");
        menu_append_item ($1, "Replace headers",  "replace_headers");
        menu_append_item ($1, "Copy msgstr",  "copy_msgstr");
        menu_append_item ($1, "Insert msgstr",  "insert_msgstr");
        menu_append_item ($1, "Copy msgid to msgstr",  "copy_msgid_to_msgstr");
        menu_append_item ($1, "Cut msgstr",  "cut_msgstr");
        menu_append_item ($1, "Flag entry as fuzzy",  "fuzzyize");
        menu_append_item ($1, "Remove fuzzy flag",  "del_fuzzy");
        menu_append_item ($1, "Delete obsolete",  "del_obsolete");
        menu_append_item ($1, "Edit comment",  "edit_comment");
        menu_append_item ($1, "Delete comment",  "del_translator_comment");
        menu_append_item ($1, "Edit entire file",  "edit_whole_buffer");
     }
   menu_append_popup (menu, "&Keys in edit buffer");
   $2 = menu + ".&Keys in edit buffer";
     {
        menu_append_item ($2, "Finish editing",  "end_edit");
        menu_append_item ($2, "Enlarge msgstr window",  "enlargewin");
        menu_append_item ($2, "Decrease msgstr window",  "decreasewin");
     }
   menu_append_popup (menu, "&Msgfmt functions");
   $3 = menu + ".&Msgfmt functions";
     {
        menu_append_item ($3, "Compile",  "po_compile");
        menu_append_item ($3, "Validate",  "po_validate_command");
     }
}

!if (keymap_p ("po_comment"))
{
   make_keymap ("po_comment");
   definekey ("enlargewin", "^[-", "po_comment");      %  esc-alt + 
   definekey ("decreasewin", "^[/", "po_comment");     %  esc-alt -
   if (NULL != getenv ("DISPLAY"))
     {
        definekey ("enlargewin", "^[+", "po_edit");
        definekey ("decreasewin", "^[-", "po_edit");
     }
   definekey ("end_edit_comment", "^I", "po_comment"); % tab
}

!if (keymap_p ("po_edit"))
{
   make_keymap ("po_edit");
   definekey ("insert_lbreak", "^[^M", "po_edit");
   definekey ("end_edit", "^I", "po_edit");          
   definekey ("enlargewin", "^[-", "po_edit");
   definekey ("decreasewin", "^[/", "po_edit");
   if (NULL != getenv ("DISPLAY"))
     {
        definekey ("enlargewin", "^[+", "po_edit");
        definekey ("decreasewin", "^[-", "po_edit");
     }
}

!if (keymap_p ("PO"))
{
   make_keymap ("PO");
   definekey ("po_undo", "_", "PO");
   definekey ("edit_comment", "#", "PO");
   definekey ("po_statistics", "=", "PO");
   definekey ("find_fuzzy", "f", "PO");
   definekey ("bfind_fuzzy", "F", "PO");
   definekey ("goto_next_entry", "+", "PO");
   definekey ("goto_previous_entry", "-", "PO");
   definekey ("po_compile", "C", "PO");
   definekey ("copy_msgstr", "w", "PO");
   definekey ("edit_whole_buffer", "E", "PO");
   definekey ("newline_seq_off", "n", "PO");
   definekey ("newline_seq_on", "N", "PO");
   definekey ("find_obsolete", "o", "PO");
   definekey ("bfind_obsolete", "O", "PO");
   definekey ("find_translated", "t", "PO");
   definekey ("bfind_translated", "T", "PO");
   definekey ("find_untranslated", "u", "PO");
   definekey ("bfind_untranslated", "U", "PO");
   definekey ("find_untranslated", " ", "PO");
   definekey ("po_validate_command", "V", "PO");
   definekey ("edit", "^M", "PO");                         % enter
   definekey ("edit", "^I", "PO");                         % tab
   definekey ("del_fuzzy", "^[^I", "PO");                  % esc + tab
   definekey ("del_obsolete", "^[[3~", "PO");              % del
   definekey ("obsolete_entry", "^?", "PO");               % backspace
   definekey_reserved ("del_translator_comment", "#", "PO");
   definekey_reserved ("copy_msgid_to_msgstr", "c", "PO");
   definekey_reserved ("fuzzyize", "f", "PO");
   definekey_reserved ("replace_headers", "h", "PO");
   definekey_reserved ("cut_msgstr", "k", "PO");
   definekey_reserved ("insert_msgstr", "y", "PO");
}

public define po_mode ()
{
   variable PO = "PO";
   set_mode (PO, 0);
   use_keymap (PO);
   use_syntax_table (PO);
   use_dfa_syntax (use_dfa);
   set_readonly (1);
   mode_set_mode_info (PO, "init_mode_menu", &po_init_menu);
   run_mode_hooks ("po_mode_hook");
   add_to_hook ("_jed_save_buffer_before_hooks", &set_date_on_exit);
}

%}}}
